'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _pg = require('pg');

var _pg2 = _interopRequireDefault(_pg);

var _timm = require('timm');

var _throttle = require('lodash/throttle');

var _throttle2 = _interopRequireDefault(_throttle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var REQUIRED_CORE_VERSION = '^3.0.0-rc.2';

var DEFAULT_CONFIG = {
  host: 'localhost',
  port: process.env.PGPORT || 5432,
  database: process.env.PGDATABASE,
  table: 'logEntries',
  user: process.env.PGUSER,
  password: process.env.PGPASSWORD,
  throttle: 200,
  colors: true
};

var LOG_SRC = 'storyboard';
var BUF_LENGTH = 2000;

// -----------------------------------------
// Listener
// -----------------------------------------

var DbPostgresListener = function () {
  function DbPostgresListener(config, _ref) {
    var hub = _ref.hub,
        mainStory = _ref.mainStory,
        chalk = _ref.chalk;

    _classCallCheck(this, DbPostgresListener);

    this.type = 'DB_POSTGRES';
    this.config = config;
    this.hub = hub;
    this.hubId = hub.getHubId();
    this.mainStory = mainStory;
    this.chalk = chalk;
    this.client = new _pg2.default.Client(config);
    this.fConnected = false;
    // Short buffer for records to be saved
    // (accumulated during the throttle period)
    this.bufRecords = [];
    var throttlePeriod = config.throttle;

    if (throttlePeriod) {
      this.saveRecords = (0, _throttle2.default)(this.saveRecords, throttlePeriod).bind(this);
    }
    // pg.on('error', err => console.log(err));
    this.client.on('error', function () {});
    this.insertQuery = 'INSERT INTO "' + this.config.table + '" (\n        "hubId", version, "fStory", "fServer",\n        "storyId", t, src, level,\n        "fRoot", title, action, parents,\n        msg, obj, "objExpanded", "objLevel", "objOptions", "objIsError"\n      )\n      VALUES (\n        $1, $2, $3, $4, $5, $6, $7, $8,\n        $9, $10, $11, $12,\n        $13, $14, $15, $16, $17, $18\n      )\n    ';
  }

  _createClass(DbPostgresListener, [{
    key: 'init',
    value: function init() {
      var _this = this;

      var config = this.config,
          mainStory = this.mainStory;

      mainStory.info(LOG_SRC, 'Connecting to PG database...', { attach: config });
      this.client.on('error', function (err) {
        _this.fConnected = false;
        mainStory.error(LOG_SRC, 'Connection to PG database is down (will stop saving logs)', { attach: err });
      });
      this.client.on('connect', function () {
        _this.fConnected = true;
        mainStory.info(LOG_SRC, 'Connected to PG database');
        _this.saveRecords();
      });
      this.client.connect(function (err) {
        if (err) {
          mainStory.error(LOG_SRC, 'Could not connect to PG database (will not save logs)', { attach: err });
        }
      });
    }
  }, {
    key: 'tearDown',
    value: function tearDown() {
      if (!this.fConnected) return;
      this.fConnected = false;
      this.client.end(function () {});
    }
  }, {
    key: 'getConfig',
    value: function getConfig() {
      return this.config;
    }
  }, {
    key: 'addToRecordBuffer',
    value: function addToRecordBuffer(records) {
      var finalRecords = records.filter(function (r) {
        return !r.signalType;
      });
      this.bufRecords = this.bufRecords.concat(finalRecords);
      if (this.bufRecords.length > BUF_LENGTH) {
        this.bufRecords = this.bufRecords.slice(-BUF_LENGTH);
      }
    }
  }, {
    key: 'saveRecords',
    value: function saveRecords() {
      var _this2 = this;

      if (!this.fConnected) return;
      var insertQuery = this.insertQuery,
          config = this.config;

      this.bufRecords.forEach(function (r) {
        var jsonObj = r.obj != null ? JSON.stringify(r.obj) : undefined;
        var msg = config.colors ? r.msg : _this2.chalk.stripColor(r.msg);
        var title = r.title;
        if (!config.colors && title) title = _this2.chalk.stripColor(title);
        _this2.client.query(insertQuery, [
        // Common to stories and logs
        r.hubId, r.version, r.fStory, r.fServer, r.storyId, new Date(r.t), r.src, r.level,
        // Stories-only
        r.fRoot, title, r.action, JSON.stringify(r.parents),
        // Logs-only
        msg, jsonObj, r.objExpanded, r.objLevel, r.objOptions, r.objIsError])
        /* eslint-disable no-console */
        .on('error', function (err) {
          return console.log(err);
        });
        /* eslint-enable no-console */
      });
      this.bufRecords.length = 0;
    }

    // -----------------------------------------
    // Main processing function
    // -----------------------------------------

  }, {
    key: 'process',
    value: function process(msg) {
      if (msg.type !== 'RECORDS') return;
      var records = msg.data;

      this.addToRecordBuffer(records);
      this.saveRecords(); // may be throttled
    }
  }]);

  return DbPostgresListener;
}();

// -----------------------------------------
// API
// -----------------------------------------


var create = function create(userConfig, context) {
  return new DbPostgresListener((0, _timm.addDefaults)(userConfig, DEFAULT_CONFIG), context);
};
create.requiredCoreVersion = REQUIRED_CORE_VERSION;

exports.default = create;